#ifndef ladder_base_H
#define ladder_base_H

#ifdef __cplusplus
extern "C" {
#endif

#include "fe.h"
#include "ladder_base_namespace.h"

extern void ladder_base(fe *, const unsigned char *);

#ifdef __cplusplus
}
#endif

#endif /* ifndef ladder_base_H */

